@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2018	PgPool Global Development Group
+ * Copyright (c) 2003-2019	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -216,6 +216,31 @@ CommandComplete(POOL_CONNECTION * frontend, POOL_CONNECTION_POOL * backend, bool
 		pool_at_command_success(frontend, backend);
 		pool_unset_query_in_progress();
 		pool_pending_message_reset_previous_message();
+
+		if (session_context->query_context == NULL)
+		{
+			elog(WARNING, "At command complete there's no query context");
+		}
+		else
+		{
+			/*
+			 * Destroy query context if it is not referenced from sent
+			 * messages and pending messages except bound to named statements
+			 * or portals.  Named statements and portals should remain until
+			 * they are explicitly closed.
+			 */
+			if (can_query_context_destroy(session_context->query_context))
+
+			{
+				POOL_SENT_MESSAGE * msg = pool_get_sent_message_by_query_context(session_context->query_context);
+
+				if (!msg || (msg && *msg->name == '\0'))
+				{
+					pool_zap_query_context_in_sent_messages(session_context->query_context);
+					pool_query_context_destroy(session_context->query_context);
+				}
+			}
+		}
 	}
 
 	return POOL_CONTINUE;
@@ -305,6 +330,68 @@ handle_query_context(POOL_CONNECTION_POOL * backend)
 			pool_unset_failed_transaction();
 			pool_unset_transaction_isolation();
 		}
+		else if (stmt->kind == 	TRANS_STMT_COMMIT)
+		{
+			/* Commit ongoing CRETAE/DROP temp table status */
+			pool_temp_tables_commit_pending();			
+		}
+		else if (stmt->kind == TRANS_STMT_ROLLBACK)
+		{
+			/* Remove ongoing CRETAE/DROP temp table status */
+			pool_temp_tables_remove_pending();
+		}
+	}
+	else if (IsA(node, CreateStmt))
+	{
+		CreateStmt *stmt = (CreateStmt *) node;
+		POOL_TEMP_TABLE_STATE	state;
+
+		/* Is this a temporary table? */
+		if (stmt->relation->relpersistence == 't')
+		{
+			if (TSTATE(backend, MASTER_NODE_ID ) == 'T')	/* Are we inside a transaction? */
+			{
+				state = TEMP_TABLE_CREATING;
+			}
+			else
+			{
+				state = TEMP_TABLE_CREATE_COMMITTED;
+			}
+			ereport(DEBUG1,
+					(errmsg("Creating temp table: %s. commit status: %d",
+							stmt->relation->relname, state)));
+			pool_temp_tables_add(stmt->relation->relname, state);
+		}
+	}
+	else if (IsA(node, DropStmt))
+	{
+		DropStmt *stmt = (DropStmt *) node;
+		POOL_TEMP_TABLE_STATE	state;
+
+		if (stmt->removeType == OBJECT_TABLE)
+		{
+			/* Loop through stmt->objects */
+			ListCell   *cell;
+			ListCell   *next;
+
+			if (TSTATE(backend, MASTER_NODE_ID ) == 'T')	/* Are we inside a transaction? */
+			{
+				state = TEMP_TABLE_DROPPING;
+			}
+			else
+			{
+				state = TEMP_TABLE_DROP_COMMITTED;
+			}
+
+			for (cell = list_head(session_context->temp_tables); cell; cell = next)
+			{
+				char *tablename = (char *)lfirst(cell);
+				ereport(DEBUG1,
+						(errmsg("Dropping temp table: %s", tablename)));
+				pool_temp_tables_delete(tablename, state);
+				next = lnext(cell);
+			}
+		}
 	}
 }
 
