@@ -341,8 +341,12 @@ pool_connection_pool_timer(POOL_CONNECTION_POOL * backend)
 			(errmsg("setting backend connection close timer"),
 			 errdetail("close time %ld", time(NULL))));
 
-	MASTER_CONNECTION(backend)->closetime = time(NULL); /* set connection close
-														 * time */
+	/* Set connection close time */
+	for (i = 0; i < NUM_BACKENDS; i++)
+	{
+		if (CONNECTION_SLOT(backend, i))
+			CONNECTION_SLOT(backend, i)->closetime = time(NULL);
+	}
 
 	if (pool_config->connection_life_time == 0)
 		return;
@@ -395,7 +399,7 @@ pool_backend_timer(void)
 	now = time(NULL);
 
 	ereport(DEBUG1,
-			(errmsg("backend timer handler called at%ld", now)));
+			(errmsg("backend timer handler called at %ld", now)));
 
 	for (i = 0; i < pool_config->max_pool; i++, p++)
 	{
