@@ -4,7 +4,7 @@
  *
  * pgpool-recovery: exec online recovery script from SELECT statement.
  *
- * Copyright (c) 2003-2018	PgPool Global Development Group
+ * Copyright (c) 2003-2019	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -27,6 +27,7 @@
 #include "executor/spi.h"
 #include "funcapi.h"
 #include "catalog/namespace.h"
+#include "catalog/pg_proc.h"
 #include "utils/syscache.h"
 #include "utils/builtins.h"		/* PostgreSQL 8.4 needs this for textout */
 #include "utils/guc.h"
@@ -79,24 +80,37 @@ pgpool_recovery(PG_FUNCTION_ARGS)
 		elog(ERROR, "must be superuser to use pgpool_recovery function");
 #endif
 
-	if (PG_NARGS() >= 5)		/* Pgpool-II 4.0 or later */
+	if (PG_NARGS() >= 6)		/* Pgpool-II 4.1 or later */
 	{
-		char	   *remote_port = DatumGetCString(DirectFunctionCall1(textout,
+		char	   *primary_port = DatumGetCString(DirectFunctionCall1(textout,
 																	  PointerGetDatum(PG_GETARG_TEXT_P(3))));
-		int			recovery_node = PG_GETARG_INT32(4);
+		int			remote_node = PG_GETARG_INT32(4);
+
+		char	   *remote_port = DatumGetCString(DirectFunctionCall1(textout,
+																	  PointerGetDatum(PG_GETARG_TEXT_P(5))));
+
+		snprintf(recovery_script, sizeof(recovery_script), "\"%s/%s\" \"%s\" \"%s\" \"%s\" \"%s\" %d \"%s\"",
+				 DataDir, script, DataDir, remote_host,
+				 remote_data_directory, primary_port, remote_node, remote_port);
+	}
+	else if (PG_NARGS() >= 5)		/* Pgpool-II 4.0 */
+	{
+		char	   *primary_port = DatumGetCString(DirectFunctionCall1(textout,
+																	  PointerGetDatum(PG_GETARG_TEXT_P(3))));
+		int			remote_node = PG_GETARG_INT32(4);
 
 		snprintf(recovery_script, sizeof(recovery_script), "\"%s/%s\" \"%s\" \"%s\" \"%s\" \"%s\" %d",
 				 DataDir, script, DataDir, remote_host,
-				 remote_data_directory, remote_port, recovery_node);
+				 remote_data_directory, primary_port, remote_node);
 	}
 	else if (PG_NARGS() >= 4)	/* Pgpool-II 3.4 - 3.7 */
 	{
-		char	   *remote_port = DatumGetCString(DirectFunctionCall1(textout,
+		char	   *primary_port = DatumGetCString(DirectFunctionCall1(textout,
 																	  PointerGetDatum(PG_GETARG_TEXT_P(3))));
 
 		snprintf(recovery_script, sizeof(recovery_script), "\"%s/%s\" \"%s\" \"%s\" \"%s\" \"%s\"",
 				 DataDir, script, DataDir, remote_host,
-				 remote_data_directory, remote_port);
+				 remote_data_directory, primary_port);
 	}
 	else
 	{
@@ -312,8 +326,10 @@ get_function_oid(const char *funcname, const char *argtype, const char *nspname)
 	 */
 	nspid = LookupExplicitNamespace(nspname, false);
 #endif
+
 	elog(DEBUG1, "get_function_oid: oid of \"%s\": %d", nspname, nspid);
 
+
 	tup = SearchSysCache(PROCNAMEARGSNSP,
 						 PointerGetDatum(funcname),
 						 PointerGetDatum(oid_v),
@@ -322,7 +338,12 @@ get_function_oid(const char *funcname, const char *argtype, const char *nspname)
 
 	if (HeapTupleIsValid(tup))
 	{
+#if defined(PG_VERSION_NUM) && (PG_VERSION_NUM >= 120000)
+		Form_pg_proc proctup = (Form_pg_proc) GETSTRUCT(tup);
+		funcid = proctup->oid;
+#else
 		funcid = HeapTupleGetOid(tup);
+#endif
 		elog(DEBUG1, "get_function_oid: oid of \"%s\": %d", funcname, funcid);
 		ReleaseSysCache(tup);
 		return funcid;
