@@ -62,7 +62,7 @@
 #include <unistd.h>
 #include <signal.h>
 #include <ctype.h>
-#ifdef HAVE_VSYSLOG
+#ifdef HAVE_SYSLOG
 #include <syslog.h>
 #endif
 #include <string.h>
@@ -126,7 +126,7 @@ extern bool redirection_done;
  */
 emit_log_hook_type emit_log_hook = NULL;
 
-#ifdef HAVE_VSYSLOG
+#ifdef HAVE_SYSLOG
 
 /*
  * Max string length to send to syslog().  Note that this doesn't count the
@@ -1442,7 +1442,7 @@ GetErrorContextStack(void)
 }
 
 
-#ifdef HAVE_VSYSLOG
+#ifdef HAVE_SYSLOG
 
 /*
  * Set or update the parameters for syslog logging
@@ -1575,7 +1575,7 @@ write_syslog(int level, const char *line)
 		syslog(level, "[%lu] %s", seq, line);
 	}
 }
-#endif							/* HAVE_VSYSLOG */
+#endif							/* HAVE_SYSLOG */
 
 #ifdef WIN32
 /*
@@ -2018,7 +2018,8 @@ log_line_prefix(StringInfo buf, const char *line_prefix, ErrorData *edata)
 					 * pool_virtual_master_db_node_id() which eventually calls
 					 * ereport() if operated in DEBUG mode.
 					 */
-					StartupPacket *sp = session ? (session->backend->slots[REAL_MASTER_NODE_ID])->sp : NULL;
+					StartupPacket *sp = (session && session->backend && (session->backend->slots[REAL_MASTER_NODE_ID]))? \
+						(session->backend->slots[REAL_MASTER_NODE_ID])->sp: NULL;
 					const char *appname = sp ? sp->application_name : "[No Connection]";
 
 					if (appname == NULL || *appname == '\0')
@@ -2179,7 +2180,7 @@ send_message_to_server_log(ErrorData *edata)
 		}
 	}
 
-#ifdef HAVE_VSYSLOG
+#ifdef HAVE_SYSLOG
 	/* Write to syslog, if enabled */
 	if (pool_config->log_destination & LOG_DESTINATION_SYSLOG)
 	{
@@ -2217,7 +2218,7 @@ send_message_to_server_log(ErrorData *edata)
 		}
 		write_syslog(syslog_level, buf.data);
 	}
-#endif							/* HAVE_VSYSLOG */
+#endif							/* HAVE_SYSLOG */
 
 	if (pool_config->log_destination & LOG_DESTINATION_STDERR)
 	{
