@@ -6,7 +6,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2014	PgPool Global Development Group
+ * Copyright (c) 2003-2015	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -50,40 +50,58 @@
 #include "pool_config.h"
 #include "auth/md5.h"
 #include "watchdog/watchdog.h"
-#include "watchdog/wd_ext.h"
+#include "watchdog/wd_lifecheck.h"
+#include "watchdog/wd_utils.h"
 
 #define MAX_BIND_TRIES 5
+/*
+ * heartbeat packet
+ */
+typedef struct
+{
+	char		from[WD_MAX_HOST_NAMELEN];
+	int			from_pgpool_port;
+	struct timeval send_time;
+	char		hash[WD_AUTH_HASH_LEN + 1];
+}			WdHbPacket;
+
 
 static RETSIGTYPE hb_sender_exit(int sig);
 static RETSIGTYPE hb_receiver_exit(int sig);
-static int hton_wd_hb_packet(WdHbPacket *to, WdHbPacket *from);
-static int ntoh_wd_hb_packet(WdHbPacket *to, WdHbPacket *from);
-static int packet_to_string_hb(WdHbPacket *pkt, char * str, int maxlen);
+static int	hton_wd_hb_packet(WdHbPacket * to, WdHbPacket * from);
+static int	ntoh_wd_hb_packet(WdHbPacket * to, WdHbPacket * from);
+static int	packet_to_string_hb(WdHbPacket * pkt, char *str, int maxlen);
 static void wd_set_reuseport(int sock);
 
+static int	wd_create_hb_send_socket(WdHbIf * hb_if);
+static int	wd_create_hb_recv_socket(WdHbIf * hb_if);
+
+static void wd_hb_send(int sock, WdHbPacket * pkt, int len, const char *destination, const int dest_port);
+static void wd_hb_recv(int sock, WdHbPacket * pkt, char *from_addr);
+
 /* create socket for sending heartbeat */
-int
-wd_create_hb_send_socket(WdHbIf *hb_if)
+static int
+wd_create_hb_send_socket(WdHbIf * hb_if)
 {
-	int sock;
-	int tos;
+	int			sock;
+	int			tos;
 
 	/* create socket */
 	if ((sock = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
 	{
 		/* socket create failed */
 		ereport(ERROR,
-			(errmsg("failed to create watchdog heartbeat sender socket"),
+				(errmsg("failed to create watchdog heartbeat sender socket"),
 				 errdetail("create socket failed with reason: \"%s\"", strerror(errno))));
 	}
 
 	/* set socket option */
 	tos = IPTOS_LOWDELAY;
-	if (setsockopt(sock, IPPROTO_IP, IP_TOS, (char *) &tos, sizeof(tos)) == -1 )
+	if (setsockopt(sock, IPPROTO_IP, IP_TOS, (char *) &tos, sizeof(tos)) == -1)
 	{
 		close(sock);
 		ereport(ERROR,
-			(errmsg("failed to create watchdog heartbeat sender socket"),
+				(errmsg("failed to create watchdog heartbeat sender socket"),
 				 errdetail("setsockopt(IP_TOS) failed with reason: \"%s\"", strerror(errno))));
 	}
 
@@ -94,38 +112,39 @@ wd_create_hb_send_socket(WdHbIf *hb_if)
 			if (geteuid() == 0) /* check root privileges */
 			{
 				struct ifreq i;
+
 				strlcpy(i.ifr_name, hb_if->if_name, sizeof(i.ifr_name));
 
 				if (setsockopt(sock, SOL_SOCKET, SO_BINDTODEVICE, &i, sizeof(i)) == -1)
 				{
 					close(sock);
 					ereport(ERROR,
-						(errmsg("failed to create watchdog heartbeat sender socket"),
+							(errmsg("failed to create watchdog heartbeat sender socket"),
 							 errdetail("setsockopt(SO_BINDTODEVICE) failed with reason: \"%s\"", strerror(errno))));
 
 				}
 				ereport(LOG,
-					(errmsg("creating socket for sending heartbeat"),
+						(errmsg("creating socket for sending heartbeat"),
 						 errdetail("bind send socket to device: %s", i.ifr_name)));
 			}
 			else
 				ereport(LOG,
-					(errmsg("creating socket for sending heartbeat"),
+						(errmsg("creating socket for sending heartbeat"),
 						 errdetail("setsockopt(SO_BINDTODEVICE) requires root privilege")));
 		}
 #else
 		ereport(LOG,
-			(errmsg("creating socket for sending heartbeat"),
+				(errmsg("creating socket for sending heartbeat"),
 				 errdetail("setsockopt(SO_BINDTODEVICE) is not available on this platform")));
 #endif
 	}
 
 	wd_set_reuseport(sock);
 	ereport(LOG,
-		(errmsg("creating socket for sending heartbeat"),
+			(errmsg("creating socket for sending heartbeat"),
 			 errdetail("set SO_REUSEPORT")));
 
- 	if (fcntl(sock, F_SETFD, FD_CLOEXEC) < 0)
+	if (fcntl(sock, F_SETFD, FD_CLOEXEC) < 0)
 	{
 		close(sock);
 		ereport(ERROR,
@@ -137,14 +156,14 @@ wd_create_hb_send_socket(WdHbIf *hb_if)
 }
 
 /* create socket for receiving heartbeat */
-int
-wd_create_hb_recv_socket(WdHbIf *hb_if)
+static int
+wd_create_hb_recv_socket(WdHbIf * hb_if)
 {
 	struct sockaddr_in addr;
-	int sock;
-	const int one = 1;
-	int bind_tries;
-	int bind_is_done;
+	int			sock;
+	const int	one = 1;
+	int			bind_tries;
+	int			bind_is_done;
 
 	memset(&(addr), 0, sizeof(addr));
 	addr.sin_family = AF_INET;
@@ -156,15 +175,15 @@ wd_create_hb_recv_socket(WdHbIf *hb_if)
 	{
 		/* socket create failed */
 		ereport(ERROR,
-			(errmsg("failed to create watchdog heartbeat receive socket"),
+				(errmsg("failed to create watchdog heartbeat receive socket"),
 				 errdetail("create socket failed with reason: \"%s\"", strerror(errno))));
 	}
 
-	if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, (char *) &one, sizeof(one)) == -1 )
+	if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, (char *) &one, sizeof(one)) == -1)
 	{
 		close(sock);
 		ereport(ERROR,
-			(errmsg("failed to create watchdog heartbeat receive socket"),
+				(errmsg("failed to create watchdog heartbeat receive socket"),
 				 errdetail("setsockopt(SO_REUSEADDR) failed with reason: \"%s\"", strerror(errno))));
 	}
 
@@ -175,28 +194,29 @@ wd_create_hb_recv_socket(WdHbIf *hb_if)
 			if (geteuid() == 0) /* check root privileges */
 			{
 				struct ifreq i;
+
 				strlcpy(i.ifr_name, hb_if->if_name, sizeof(i.ifr_name));
 
 				if (setsockopt(sock, SOL_SOCKET, SO_BINDTODEVICE, &i, sizeof(i)) == -1)
 				{
 					close(sock);
 					ereport(ERROR,
-						(errmsg("failed to create watchdog heartbeat receive socket"),
+							(errmsg("failed to create watchdog heartbeat receive socket"),
 							 errdetail("setsockopt(SO_BINDTODEVICE) failed with reason: \"%s\"", strerror(errno))));
 				}
 				ereport(LOG,
-					(errmsg("createing watchdog heartbeat receive socket."),
+						(errmsg("createing watchdog heartbeat receive socket."),
 						 errdetail("bind receive socket to device: \"%s\"", i.ifr_name)));
 
 			}
 			else
 				ereport(LOG,
-					(errmsg("failed to create watchdog heartbeat receive socket."),
+						(errmsg("failed to create watchdog heartbeat receive socket."),
 						 errdetail("setsockopt(SO_BINDTODEVICE) requies root privilege")));
 		}
 #else
 		ereport(LOG,
-			(errmsg("failed to create watchdog heartbeat receive socket"),
+				(errmsg("failed to create watchdog heartbeat receive socket"),
 				 errdetail("setsockopt(SO_BINDTODEVICE) is not available on this platform")));
 #endif
 	}
@@ -204,18 +224,18 @@ wd_create_hb_recv_socket(WdHbIf *hb_if)
 	wd_set_reuseport(sock);
 
 	ereport(LOG,
-		(errmsg("creating watchdog heartbeat receive socket."),
+			(errmsg("creating watchdog heartbeat receive socket."),
 			 errdetail("set SO_REUSEPORT")));
 
 	bind_is_done = 0;
 	for (bind_tries = 0; !bind_is_done && bind_tries < MAX_BIND_TRIES; bind_tries++)
 	{
-		if (bind(sock, (struct sockaddr *)&addr, sizeof(struct sockaddr)) < 0)
+		if (bind(sock, (struct sockaddr *) &addr, sizeof(struct sockaddr)) < 0)
 		{
 			ereport(LOG,
-				(errmsg("failed to create watchdog heartbeat receive socket. retrying..."),
-					errdetail("bind failed with reason: \"%s\"",
-						   strerror(errno))));
+					(errmsg("failed to create watchdog heartbeat receive socket. retrying..."),
+					 errdetail("bind failed with reason: \"%s\"",
+							   strerror(errno))));
 
 			sleep(1);
 		}
@@ -230,15 +250,15 @@ wd_create_hb_recv_socket(WdHbIf *hb_if)
 	{
 		close(sock);
 		ereport(ERROR,
-			(errmsg("failed to create watchdog heartbeat receive socket"),
+				(errmsg("failed to create watchdog heartbeat receive socket"),
 				 errdetail("bind socket failed with reason: \"%s\"", strerror(errno))));
 	}
 
- 	if (fcntl(sock, F_SETFD, FD_CLOEXEC) < 0)
+	if (fcntl(sock, F_SETFD, FD_CLOEXEC) < 0)
 	{
 		close(sock);
 		ereport(ERROR,
-			(errmsg("failed to create watchdog heartbeat receive socket"),
+				(errmsg("failed to create watchdog heartbeat receive socket"),
 				 errdetail("setting close-on-exec flag failed with reason: \"%s\"", strerror(errno))));
 	}
 
@@ -246,23 +266,23 @@ wd_create_hb_recv_socket(WdHbIf *hb_if)
 }
 
 /* send heartbeat signal */
-void
-wd_hb_send(int sock, WdHbPacket * pkt, int len, const char * host, const int port)
+static void
+wd_hb_send(int sock, WdHbPacket * pkt, int len, const char *host, const int port)
 {
-	int rtn;
+	int			rtn;
 	struct sockaddr_in addr;
 	struct hostent *hp;
-	WdHbPacket buf;
+	WdHbPacket	buf;
 
 	if (!host || !strlen(host))
 		ereport(ERROR,
-			(errmsg("failed to send watchdog heartbeat. host name is empty")));
+				(errmsg("failed to send watchdog heartbeat. host name is empty")));
 
 	hp = gethostbyname(host);
 	if ((hp == NULL) || (hp->h_addrtype != AF_INET))
 		ereport(ERROR,
-			(errmsg("failed to send watchdog heartbeat, gethostbyname() failed"),
-				 errdetail("gethostbyname on \"%s\" failed with reason: \"%s\"",host, hstrerror(h_errno))));
+				(errmsg("failed to send watchdog heartbeat, gethostbyname() failed"),
+				 errdetail("gethostbyname on \"%s\" failed with reason: \"%s\"", host, hstrerror(h_errno))));
 
 	memmove((char *) &(addr.sin_addr), (char *) hp->h_addr, hp->h_length);
 
@@ -272,33 +292,38 @@ wd_hb_send(int sock, WdHbPacket * pkt, int len, const char * host, const int por
 	hton_wd_hb_packet(&buf, pkt);
 
 	if ((rtn = sendto(sock, &buf, sizeof(WdHbPacket), 0,
-	                  (struct sockaddr *)&addr, sizeof(addr))) != len)
+					  (struct sockaddr *) &addr, sizeof(addr))) != len)
 	{
 		ereport(ERROR,
-			(errmsg("failed to send watchdog heartbeat, sendto failed"),
-				 errdetail("sending packet to \"%s\" failed with reason: \"%s\"",host, strerror(errno))));
+				(errmsg("failed to send watchdog heartbeat, sendto failed"),
+				 errdetail("sending packet to \"%s\" failed with reason: \"%s\"", host, strerror(errno))));
 
 	}
 	ereport(DEBUG2,
 			(errmsg("watchdog heartbeat: send %d byte packet", rtn)));
+
 }
 
-/* receive heartbeat signal */
+/*
+ * receive heartbeat signal
+ * the function expects the from_addr to be at least WD_MAX_HOST_NAMELEN bytes long
+ */
 void
-wd_hb_recv(int sock, WdHbPacket * pkt)
+static
+wd_hb_recv(int sock, WdHbPacket * pkt, char *from_addr)
 {
-	int rtn;
+	int			rtn;
 	struct sockaddr_in senderinfo;
-	socklen_t addrlen;
-	WdHbPacket buf;
+	socklen_t	addrlen;
+	WdHbPacket	buf;
 
 	addrlen = sizeof(senderinfo);
 
 	rtn = recvfrom(sock, &buf, sizeof(WdHbPacket), 0,
-	               (struct sockaddr *)&senderinfo, &addrlen);
+				   (struct sockaddr *) &senderinfo, &addrlen);
 	if (rtn < 0)
 		ereport(ERROR,
-			(errmsg("failed to receive heartbeat packet")));
+				(errmsg("failed to receive heartbeat packet")));
 	else if (rtn == 0)
 		ereport(ERROR,
 				(errmsg("failed to receive heartbeat received zero length packet")));
@@ -306,27 +331,26 @@ wd_hb_recv(int sock, WdHbPacket * pkt)
 		ereport(DEBUG2,
 				(errmsg("watchdog heartbeat: received %d byte packet", rtn)));
 
+	strncpy(from_addr, inet_ntoa(senderinfo.sin_addr), WD_MAX_HOST_NAMELEN - 1);
+
 	ntoh_wd_hb_packet(pkt, &buf);
 }
 
 /* fork heartbeat receiver child */
 pid_t
-wd_hb_receiver(int fork_wait_time, WdHbIf *hb_if)
+wd_hb_receiver(int fork_wait_time, WdHbIf * hb_if)
 {
-	int sock;
-	pid_t pid = 0;
-	WdHbPacket pkt;
+	int			sock;
+	pid_t		pid = 0;
+	WdHbPacket	pkt;
 	struct timeval tv;
-	char from[WD_MAX_HOST_NAMELEN];
-	int from_pgpool_port;
-	char buf[(MD5_PASSWD_LEN+1)*2];
-	char pack_str[WD_MAX_PACKET_STRING];
-	int pack_str_len;
+	char		from[WD_MAX_HOST_NAMELEN];
+	int			from_pgpool_port;
+	char		buf[WD_AUTH_HASH_LEN + 1];
+	char		pack_str[WD_MAX_PACKET_STRING];
+	int			pack_str_len;
 	sigjmp_buf	local_sigjmp_buf;
 
-
-	WdInfo * p;
-
 	pid = fork();
 	if (pid != 0)
 	{
@@ -363,7 +387,7 @@ wd_hb_receiver(int fork_wait_time, WdHbIf *hb_if)
 											   ALLOCSET_DEFAULT_MINSIZE,
 											   ALLOCSET_DEFAULT_INITSIZE,
 											   ALLOCSET_DEFAULT_MAXSIZE);
-	
+
 	MemoryContextSwitchTo(TopMemoryContext);
 
 	sock = wd_create_hb_recv_socket(hb_if);
@@ -374,32 +398,38 @@ wd_hb_receiver(int fork_wait_time, WdHbIf *hb_if)
 	{
 		/* Since not using PG_TRY, must reset error stack by hand */
 		error_context_stack = NULL;
-		
+
 		EmitErrorReport();
 		MemoryContextSwitchTo(TopMemoryContext);
 		FlushErrorState();
 	}
-	
+
 	/* We can now handle ereport(ERROR) */
 	PG_exception_stack = &local_sigjmp_buf;
 
-	for(;;)
+	for (;;)
 	{
+		int			i;
+
 		MemoryContextSwitchTo(ProcessLoopContext);
 		MemoryContextResetAndDeleteChildren(ProcessLoopContext);
 
 		/* receive heartbeat signal */
-		wd_hb_recv(sock, &pkt);
-			/* authentication */
+		wd_hb_recv(sock, &pkt, from);
+		/* authentication */
 		if (strlen(pool_config->wd_authkey))
 		{
 			/* calculate hash from packet */
 			pack_str_len = packet_to_string_hb(&pkt, pack_str, sizeof(pack_str));
 			wd_calc_hash(pack_str, pack_str_len, buf);
 
+			if (buf[0] == '\0')
+				ereport(WARNING,
+						(errmsg("failed to calculate wd_authkey hash from a received heartbeat packet")));
+
 			if (strcmp(pkt.hash, buf))
 				ereport(ERROR,
-					(errmsg("watchdog heartbeat receive"),
+						(errmsg("watchdog heartbeat receive"),
 						 errdetail("authentication failed")));
 		}
 
@@ -407,44 +437,36 @@ wd_hb_receiver(int fork_wait_time, WdHbIf *hb_if)
 		gettimeofday(&tv, NULL);
 
 		/* who send this packet? */
-		strlcpy(from, pkt.from, sizeof(from));
 		from_pgpool_port = pkt.from_pgpool_port;
-
-		p = WD_List;
-		while (p->status != WD_END)
+		for (i = 0; i < gslifeCheckCluster->nodeCount; i++)
 		{
-			if (!strcmp(p->hostname, from) && p->pgpool_port == from_pgpool_port)
+			LifeCheckNode *node = &gslifeCheckCluster->lifeCheckNodes[i];
+
+			ereport(DEBUG2,
+					(errmsg("received heartbeat signal from \"%s:%d\" hostname:%s",
+							from, from_pgpool_port, pkt.from)));
+
+			if ((!strcmp(node->hostName, pkt.from) || !strcmp(node->hostName, from)) && node->pgpoolPort == from_pgpool_port)
 			{
-				/* ignore the packet from down pgpool */
-				if (pkt.status == WD_DOWN)
-				{
-					ereport(DEBUG1,
-						(errmsg("watchdog heartbeat: received heartbeat signal from \"%s:%d\" whose status is down. ignored",
-									from, from_pgpool_port)));
-					break;
-				}
-
 				/* this is the first packet or the latest packet */
-				if (!WD_TIME_ISSET(p->hb_send_time) ||
-					WD_TIME_BEFORE(p->hb_send_time, pkt.send_time))
+				if (!WD_TIME_ISSET(node->hb_send_time) ||
+					WD_TIME_BEFORE(node->hb_send_time, pkt.send_time))
 				{
 					ereport(DEBUG1,
-							(errmsg("watchdog heartbeat: received heartbeat signal from \"%s:%d\"",
-									from, from_pgpool_port)));
+							(errmsg("received heartbeat signal from \"%s(%s):%d\" node:%s",
+									from, pkt.from, from_pgpool_port, node->nodeName)));
 
-					p->hb_send_time = pkt.send_time;
-					p->hb_last_recv_time = tv;
+					node->hb_send_time = pkt.send_time;
+					node->hb_last_recv_time = tv;
 				}
 				else
 				{
-					ereport(DEBUG1,
-							(errmsg("watchdog heartbeat: received heartbeat signal is older than the latest, ignored")));
+					ereport(NOTICE,
+							(errmsg("received heartbeat signal is older than the latest, ignored")));
 				}
 				break;
 			}
-			p++;
 		}
-
 	}
 
 	return pid;
@@ -452,14 +474,14 @@ wd_hb_receiver(int fork_wait_time, WdHbIf *hb_if)
 
 /* fork heartbeat sender child */
 pid_t
-wd_hb_sender(int fork_wait_time, WdHbIf *hb_if)
+wd_hb_sender(int fork_wait_time, WdHbIf * hb_if)
 {
-	int sock;
-	pid_t pid = 0;
-	WdHbPacket pkt;
-	WdInfo * p = WD_List;
-	char pack_str[WD_MAX_PACKET_STRING];
-	int pack_str_len;
+	int			sock;
+	pid_t		pid = 0;
+	WdHbPacket	pkt;
+
+	char		pack_str[WD_MAX_PACKET_STRING];
+	int			pack_str_len;
 	sigjmp_buf	local_sigjmp_buf;
 
 	pid = fork();
@@ -498,7 +520,7 @@ wd_hb_sender(int fork_wait_time, WdHbIf *hb_if)
 											   ALLOCSET_DEFAULT_MINSIZE,
 											   ALLOCSET_DEFAULT_INITSIZE,
 											   ALLOCSET_DEFAULT_MAXSIZE);
-	
+
 	MemoryContextSwitchTo(TopMemoryContext);
 
 	sock = wd_create_hb_send_socket(hb_if);
@@ -509,17 +531,17 @@ wd_hb_sender(int fork_wait_time, WdHbIf *hb_if)
 	{
 		/* Since not using PG_TRY, must reset error stack by hand */
 		error_context_stack = NULL;
-		
+
 		EmitErrorReport();
 		MemoryContextSwitchTo(TopMemoryContext);
 		FlushErrorState();
 		sleep(pool_config->wd_heartbeat_keepalive);
 	}
-	
+
 	/* We can now handle ereport(ERROR) */
 	PG_exception_stack = &local_sigjmp_buf;
 
-	for(;;)
+	for (;;)
 	{
 		MemoryContextSwitchTo(ProcessLoopContext);
 		MemoryContextResetAndDeleteChildren(ProcessLoopContext);
@@ -529,14 +551,16 @@ wd_hb_sender(int fork_wait_time, WdHbIf *hb_if)
 		strlcpy(pkt.from, pool_config->wd_hostname, sizeof(pkt.from));
 		pkt.from_pgpool_port = pool_config->port;
 
-		pkt.status = p->status;
-
 		/* authentication key */
 		if (strlen(pool_config->wd_authkey))
 		{
 			/* calculate hash from packet */
 			pack_str_len = packet_to_string_hb(&pkt, pack_str, sizeof(pack_str));
 			wd_calc_hash(pack_str, pack_str_len, pkt.hash);
+
+			if (pkt.hash[0] == '\0')
+				ereport(WARNING,
+						(errmsg("failed to calculate wd_authkey hash from a heartbeat packet to be sent")));
 		}
 
 		/* send heartbeat signal */
@@ -554,17 +578,17 @@ hb_sender_exit(int sig)
 {
 	switch (sig)
 	{
-		case SIGTERM:	/* smart shutdown */
-		case SIGINT:	/* fast shutdown */
-		case SIGQUIT:	/* immediate shutdown */
+		case SIGTERM:			/* smart shutdown */
+		case SIGINT:			/* fast shutdown */
+		case SIGQUIT:			/* immediate shutdown */
 			ereport(DEBUG1,
-					(errmsg("watchdog heartbeat sender child receives shutdown request signal %d", sig )));
+					(errmsg("watchdog heartbeat sender child receives shutdown request signal %d", sig)));
 			break;
 		default:
 			ereport(LOG,
-					(errmsg("hb_sender child receives unknown signal: %d", sig )));
+					(errmsg("hb_sender child receives unknown signal: %d", sig)));
 	}
-	
+
 	exit(0);
 }
 
@@ -573,17 +597,17 @@ hb_receiver_exit(int sig)
 {
 	switch (sig)
 	{
-		case SIGTERM:	/* smart shutdown */
-		case SIGINT:	/* fast shutdown */
-		case SIGQUIT:	/* immediate shutdown */
+		case SIGTERM:			/* smart shutdown */
+		case SIGINT:			/* fast shutdown */
+		case SIGQUIT:			/* immediate shutdown */
 			ereport(DEBUG1,
-				(errmsg("watchdog heartbeat receiver child receives shutdown request signal %d", sig )));
+					(errmsg("watchdog heartbeat receiver child receives shutdown request signal %d", sig)));
 			break;
 		default:
 			ereport(LOG,
-					(errmsg("hb_receiver child receives unknown signal: %d", sig )));
+					(errmsg("hb_receiver child receives unknown signal: %d", sig)));
 	}
-	
+
 	exit(0);
 }
 
@@ -595,7 +619,6 @@ hton_wd_hb_packet(WdHbPacket * to, WdHbPacket * from)
 		return WD_NG;
 	}
 
-	to->status = htonl(from->status);
 	to->send_time.tv_sec = htonl(from->send_time.tv_sec);
 	to->send_time.tv_usec = htonl(from->send_time.tv_usec);
 	memcpy(to->from, from->from, sizeof(to->from));
@@ -613,7 +636,6 @@ ntoh_wd_hb_packet(WdHbPacket * to, WdHbPacket * from)
 		return WD_NG;
 	}
 
-	to->status = ntohl(from->status);
 	to->send_time.tv_sec = ntohl(from->send_time.tv_sec);
 	to->send_time.tv_usec = ntohl(from->send_time.tv_usec);
 	memcpy(to->from, from->from, sizeof(to->from));
@@ -625,11 +647,12 @@ ntoh_wd_hb_packet(WdHbPacket * to, WdHbPacket * from)
 
 /* convert packet to string and return length of the string */
 static int
-packet_to_string_hb(WdHbPacket *pkt, char *str, int maxlen)
+packet_to_string_hb(WdHbPacket * pkt, char *str, int maxlen)
 {
-	int len;
-	len = snprintf(str, maxlen, "status=%d tv_sec=%ld tv_usec=%ld from=%s",
-	               pkt->status, pkt->send_time.tv_sec, pkt->send_time.tv_usec, pkt->from);
+	int			len;
+
+	len = snprintf(str, maxlen, "tv_sec=%ld tv_usec=%ld from=%s",
+				   pkt->send_time.tv_sec, pkt->send_time.tv_usec, pkt->from);
 
 	return len;
 }
@@ -639,17 +662,18 @@ packet_to_string_hb(WdHbPacket *pkt, char *str, int maxlen)
  * in the compile time but not available in the run time, just emit a
  * log and treat it as normal.
  */
-static void wd_set_reuseport(int sock)
+static void
+wd_set_reuseport(int sock)
 {
 #if defined(SO_REUSEPORT)
-	int one = 1;
+	int			one = 1;
 
 	if (setsockopt(sock, SOL_SOCKET, SO_REUSEPORT, &one, sizeof(one)) == -1)
 	{
 		if (errno == EINVAL || errno == ENOPROTOOPT)
 		{
 			ereport(LOG,
-				(errmsg("error seting SO_REUSEPORT option to the socket"),
+					(errmsg("error seting SO_REUSEPORT option to the socket"),
 					 errdetail("setsockopt(SO_REUSEPORT) is not supported by the kernel. detail: %s",
 							   strerror(errno))));
 		}
@@ -657,7 +681,7 @@ static void wd_set_reuseport(int sock)
 		{
 			close(sock);
 			ereport(ERROR,
-				(errmsg("failed to create watchdog heartbeat socket"),
+					(errmsg("failed to create watchdog heartbeat socket"),
 					 errdetail("setsockopt(SO_REUSEPORT) failed with reason: \"%s\"", strerror(errno))));
 		}
 	}
